/* eslint-disable global-require */
import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/try-catch-remove/input.js?commonjs-exports"

try {
  
} catch (ignored) {
  /* ignore */
}

export default input;
export { input as __moduleExports };
