import { ITest } from './interface';

// eslint-disable-next-line import/prefer-default-export
export { ITest };

const answer = 42;
// eslint-disable-next-line no-console
console.log(`the answer is ${answer}`);
