const dep = require('./dep.js');

function _interopDefault(e) {
  return e && e.__esModule ? e : { default: e };
}

t.is(dep.__esModule, true);
const dep__default = /* #__PURE__*/ _interopDefault(dep);
t.is(dep__default.default, 'default');
