import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/compiled-esm-define-exports-empty/input.js?commonjs-exports"

Object.defineProperty(input, "__esModule", { value: true });

export { input as __moduleExports, input as default };
