import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/compiled-esm-minified/input.js?commonjs-exports"

Object.defineProperty(input, '__esModule', { value: !0 });
var foo = input.foo = 'foo';

export { input as __moduleExports, foo, input as default };
