import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/dynamic-template-literal/input.js?commonjs-exports"

var pe = 'pe';
var foo = commonjsHelpers.commonjsRequire(`ta${pe}`);
console.log(foo);

export default input;
export { input as __moduleExports };
