import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/compiled-esm-deconflict/input.js?commonjs-exports"

Object.defineProperty(input, '__esModule', { value: true });
var foo_1 = input.foo = 'bar';

const foo = 'also bar';

export { input as __moduleExports, foo_1 as foo, input as default };
