const lib = require('../');

module.exports.extend1 = function (exports) {
  exports.get2 = function () {
    return 'indirect ref';
  };
};

module.exports.extend2 = function (exports) {
  exports.get3 = lib.get1;
};
