const identifier = require('./identifier');
const property = require('./property');
const arrayPattern = require('./array-pattern.js');
const assignmentPattern = require('./assignment-pattern.js');

t.deepEqual(identifier, {});
t.deepEqual(property, {});
t.deepEqual(arrayPattern, {});
t.deepEqual(assignmentPattern, {});
