import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/optimised-named-export/input.js?commonjs-exports"

var foo = input.foo = 'a';
var bar = input.bar = 'b';

export default input;
export { input as __moduleExports, foo, bar };
