import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/unambiguous-with-import/input.js?commonjs-exports"
import "\u0000./foo.js?commonjs-require";
import "\u0000./foo.js?commonjs-proxy";

import './bar.js';
