module.exports = {
  description:
    'handles requiring an ES module that reexports a namespace when returning the default is preferred',
  pluginOptions: {
    requireReturnsDefault: 'preferred'
  }
};
