import * as commonjsHelpers from "_commonjsHelpers.js";

var foo = function () {
	return;
};

var bar = () => {
	return;
};

function baz () {
	return;
}

var input = 42;

export default input;
export { input as __moduleExports };
