import { Plugin } from 'rollup';

export interface RollupVirtualOptions {
  [id: string]: string;
}

/**
 * A Rollup plugin which loads virtual modules from memory.
 */
export default function virtual(modules: RollupVirtualOptions): Plugin;
