import a1 from 'exports-directory-specificity/one/a.js';
import a2 from 'exports-directory-specificity/one/two/a.js';
import a3 from 'exports-directory-specificity/one/two/three/a.js';

export default { a1, a2, a3 };
