import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/ignore-ids/input.js?commonjs-exports"
import "\u0000bar?commonjs-require";
import require$$0 from "\u0000bar?commonjs-proxy";

var foo = require( 'foo' );
var bar = require$$0;

export default input;
export { input as __moduleExports };
