import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/compiled-esm-define-exports/input.js?commonjs-exports"

Object.defineProperty(input, '__esModule', { value: true });
var _default = input.default = 'x';
var foo = input.foo = 'foo';

export { input as __moduleExports, foo, _default as default };
