import * as commonjsHelpers from "_commonjsHelpers.js";
import { __exports as input } from "\u0000fixtures/form/multiple-var-declarations-c/input.js?commonjs-exports"
import "\u0000./b?commonjs-require";
import require$$0 from "\u0000./b?commonjs-proxy";

var a = 'a'
  , b = require$$0
  , c = 'c';

console.log( a, b, c );

export default input;
export { input as __moduleExports };
